import hashlib
import platform
import socket


__all__ = [
    "windows_codepage_cp1252",
    "generate_authtoken",
    "enable_socket_keepalive",
]


def windows_codepage_cp1252(error):
    """Windows Codepage 1252 decoder fallback

    This function could be used as a failback for UnicodeEncodeError to fail
    back to windows codepage 1252 decoding when eg. utf-8 failes

    All valid windows codepage 1252 elements will be returned with their
    representative unicode characters and failes back to unicode unknown char
    when no valid character is found

    All illegal characters are replaced with unicode 0xFFFD
    """

    # This only works on UnicodeDecodeError, will not work on encoding
    if not isinstance(error, UnicodeDecodeError):
        raise error

    # 0xFFFD is the unicode char for illegal character.
    # these characters are not an valid printable cp1252 character

    cp1252_map = [
        0x20AC, 0xFFFD, 0x201A, 0x0192, 0x201E, 0x2026, 0x2020, 0x2021,  # 0x80-0x87
        0x02c6, 0x2030, 0x0160, 0x2039, 0x0152, 0xFFFD, 0x017d, 0xFFFD,  # 0x88-0x8F
        0xFFFD, 0x2018, 0x2019, 0x201C, 0x201D, 0x2022, 0x2013, 0x2014,  # 0x90-0x97
        0x02DC, 0x2122, 0x0161, 0x203A, 0x0153, 0xFFFD, 0x017E, 0x0178   # 0x98-0x9F
    ]  # fmt: skip

    result = []
    for i in range(error.start, error.end):
        byte = error.object[i]

        if byte >= 0x80 and byte <= 0x9F:
            # We try to use windows codepage 1252 on this char
            result.append(chr(cp1252_map[byte - 0x80]))

        else:
            # This looks like a valid latin-1 char
            # It correspons to the same unicode char
            result.append(chr(0x00 + byte))

    return "".join(result), error.end


def generate_authtoken(challenge, password):
    "Combine Password and authChallenge from Ritz to make authToken"

    raw_token = "%s %s" % (challenge, password)
    token = hashlib.sha1(raw_token.encode("UTF-8")).hexdigest()
    return token


def _enable_keepalive_linux(sock, after_idle_sec, interval_sec, max_fails):
    """Set TCP keepalive on an open socket.

    It activates after 1 second (after_idle_sec) of idleness,
    then sends a keepalive ping once every 3 seconds (interval_sec),
    and closes the connection after 5 failed ping (max_fails), or 15 seconds
    """
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
    sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, after_idle_sec)
    sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, interval_sec)
    sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPCNT, max_fails)


def _enable_keepalive_osx(sock, after_idle_sec, interval_sec, max_fails):
    """Set TCP keepalive on an open socket.

    sends a keepalive ping once every 3 seconds (interval_sec)
    """
    # scraped from /usr/include, not exported by python's socket module
    TCP_KEEPALIVE = 0x10
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
    sock.setsockopt(socket.IPPROTO_TCP, TCP_KEEPALIVE, interval_sec)


def _enable_keepalive_win(sock, after_idle_sec, interval_sec, max_fails):
    sock.ioctl(socket.SIO_KEEPALIVE_VALS, (1, after_idle_sec * 1000, interval_sec * 1000))


def enable_socket_keepalive(sock, after_idle_sec=60, interval_sec=60, max_fails=5):
    platforms = {
        "Linux": _enable_keepalive_linux,
        "Darwin": _enable_keepalive_osx,
        "Windows": _enable_keepalive_win,
    }
    plat = platform.system()
    if plat in platforms:
        return platforms[plat](sock, after_idle_sec=60, interval_sec=60, max_fails=5)
    raise RuntimeError('Unsupported platform: {}'.format(plat))



from ritz import ritz, notifier, parse_tcl_config, caseType
from pprint import pprint
from os.path import expanduser
from time import sleep
import re
import argparse
import sys
import logging


cases = {}



def show():
  print(chr(27) + "[2J")      # Clear screen
  for c in cases:
    case = cases[c]
    if case["type"] == caseType.PORTSTATE:
      # LowerLayerDown state is quite long... shorten it
      if "lowerLayerDown" in case["portstate"]:
        portState = "LowerDown"
      else:
        portState = case["portstate"]

      print("%-10s %-8s %-15s %-24s %-s" % (
            portState, case["state"], case["router"],
            case["port"], case["descr"] if "descr" in case else "---"))

def main():
  parser = argparse.ArgumentParser(description='Process some integers.')

  parser.add_argument('--prod', action='store_true')

  args = parser.parse_args()
  conf = parse_tcl_config("~/.ritz.tcl")

  if args.prod:
    c_server = conf["default"]["Server"]
    c_user   = conf["default"]["User"]
    c_secret = conf["default"]["Secret"]
  else:
    c_server = conf["UNINETT-backup"]["Server"]
    c_user   = conf["UNINETT-backup"]["User"]
    c_secret = conf["UNINETT-backup"]["Secret"]

  ritzlog = logging.getLogger("ritz")
  ritzlog.setLevel(logging.DEBUG)
  ritzlog.addHandler(logging.FileHandler('comm.log'))
  sess = ritz(c_server)
  sess.connect()
  sess.authenticate(c_user, c_secret)
  get_caseids = sess.get_caseids()

  for i in get_caseids:
    print("Case: %i" % i)
    cases[i] = sess.get_attributes(i)

  show()

  pass
  notif = notifier(sess)
  notif.connect()
  while True:
    n = notif.poll()
    if n:
      print(n)
      p = n.split(' ', 2)
      if "attr" in p[1]:
        pprint(sess.get_attributes(int(p[0])))
      elif "log" in p[1]:
        pprint(sess.get_log(int(p[0])))
      elif "state" in p[1]:
        v = p[2].split(' ', 1)
        print("State on %s changed from %s to %s" % (p[0], v[0], v[1]))
      else:
        print("UNKNOWN!!!!  : %s" % n)

      print()
    sleep(1)


  if args.CaseID not in get_caseids:
    print(get_caseids)
    print("CaseID %i is not in database" % args.CaseID)
    sys.exit(1)

  pprint(sess.get_attributes(args.CaseID))
  print("Get History: %i" % args.CaseID)
  pprint(sess.gethist(args.CaseID))
  print("Get Log: %i" % args.CaseID)
  pprint(sess.get_log(args.CaseID))
  # print("Add test message to log: 24014")
  # pprint(sess.add_history(24014, "Testmelding ifra pyRitz"))
  # print("set_state 'open': %i" % args.CaseID)
  # pprint(sess.set_state(args.CaseID, "open"))


if __name__ == "__main__":
  main()



from ritz import ritz, notifier, parse_tcl_config
from pprint import pprint
from os.path import expanduser
from time import sleep
import re
import argparse
import sys


def importconf(file):
  config = {}
  with open(expanduser(file), "r") as f:
    for line in f.readlines():
      sets = re.findall("^\s?set\s+(\S+)\s+(.*)$", line)
      if sets:
        config[sets[0][0]] = sets[0][1]
  return config

def main():
  parser = argparse.ArgumentParser(description='Process some integers.')

  parser.add_argument('CaseID', metavar='N', type=int,
                      help='CaseID to test with')
  parser.add_argument('--prod', action='store_true')

  args = parser.parse_args()
  conf = parse_tcl_config("~/.ritz.tcl")

  if args.prod:
    c_server = conf["default"]["Server"]
    c_user   = conf["default"]["User"]
    c_secret = conf["default"]["Secret"]
  else:
    c_server = conf["UNINETT-backup"]["Server"]
    c_user   = conf["UNINETT-backup"]["User"]
    c_secret = conf["UNINETT-backup"]["Secret"]
  with ritz(c_server, username=c_user, password=c_secret) as sess:
    get_caseids = sess.get_caseids()
    if args.CaseID == 0:
      for i in get_caseids:
        print("Case: %i" % i)
        pprint(sess.get_attributes(i))
    elif args.CaseID == 1:
      notif = notifier()
      key = notif.connect(c_server)
      sess.ntie(key)
      while True:
        n = notif.poll()
        if n:
          print n
          p = n.split(' ',2)
          if "attr" in p[1]:
            pprint(sess.get_attributes(int(p[0])))
          elif "log" in p[1]:
            pprint(sess.get_log(int(p[0])))
          elif "state" in p[1]:
            v = p[2].split(' ', 1)
            print("State on %s changed from %s to %s" % (p[0],v[0],v[1]))
          else:
            print("UNKNOWN!!!!  : %s" % n)

          print()
        sleep(1)
    else:
      if args.CaseID not in get_caseids:
        print(get_caseids)
        print("CaseID %i is not in database" % args.CaseID)
        sys.exit(1)

      pprint(sess.get_attributes(args.CaseID))
      print("Get History: %i" % args.CaseID)
      pprint(sess.get_history(args.CaseID))
      print("Get Log: %i" % args.CaseID)
      pprint(sess.get_log(args.CaseID))
      #print("Add test message to log: 24014")
      #pprint(sess.add_history(24014, "Testmelding ifra pyRitz"))
      #print("set_state 'open': %i" % args.CaseID)
      #pprint(sess.set_state(args.CaseID, "open"))
if __name__ == "__main__":
  main()



import subprocess
import logging
import re
import sys
import os
import socket
from ritz import ritz, caseType, caseState
from datetime import datetime, timedelta
import traceback
import re



def main():
    """
    Script to schedule downtime of interfaces in zino when a server reboots.

    This script tries to detect interface names via LLDP and uses the env
    variables ZINOUSER and ZINOSECRET to connect to zino as as a ritz client.
    """
    if "ZINODEBUG" in os.environ:
        ritzlog = logging.getLogger("ritz")
        ritzlog.setLevel(logging.DEBUG)
        ritzlog.addHandler(logging.FileHandler('comm.log'))

    if "ZINOUSER" in os.environ and "ZINOSECRET" in os.environ:
        user = os.environ["ZINOUSER"]
        secret = os.environ["ZINOSECRET"]
    else:
        sys.exit("ZINOUSER and/or ZINOSECRET env variable is not set, exiting")

    if "ZINOSERVER" in os.environ:
        server = os.environ["ZINOSERVER"]
    else:
        server = "zino.uninett.no"
    if "ZINOHOSTNAME" in os.environ:
        fqdn = os.environ["ZINOHOSTNAME"]
    else:
        fqdn = socket.getfqdn()

    if not fqdn.endswith(".uninett.no"):
        sys.exit("Wooth? sysname does not end with uninett.no?")

    print("Removing Maintenance on physical interfaces")
    with ritz(server, username=user, password=secret) as s:
        try:
            # Actually searching for ', fqdn.domain.ext'

            ids = s.get_caseids()
            for id in ids:
                try:
                    attr = s.case(id)
                except Exception as e:
                    print(e)
                    #print("Invalid case {id}".format(id=id))
                    continue
                if attr.type not in [caseType.PORTSTATE]:
                    print("case {id} is not a portstate".format(**attr))
                    continue

                #print("Checking case {id}: {descr}".format(**attr))
                #Check if case has a descr
                if not attr.has_key("descr"):
                    continue

                if not re.search(r", {}".format(fqdn.replace(".", r"\.")), attr.descr):
                    print("Did not match: {}".format(attr.descr))
                    continue

                print("Found case {id}: {descr}".format(**attr))
                if attr.portstate == "up" and attr.state in [caseState.IGNORED, caseState.OPEN]:
                  # This case is not tampered with, just close it :)
                  try:
                      s.clear_flapping(attr.router, attr.ifindex)

                      s.add_history(id, "This case is automatically closed by {filename}".format(filename=os.path.basename(__file__)))
                      s.set_state(id, "closed")
                      print("  Closed")
                  except Exception as e:
                    print("  Error closing: %s" % e)

        except Exception as e:
            print("Unable to clean : %s" % (traceback.format_exc()))

if __name__ == "__main__":
    main()



from ritz import ritz, notifier, parse_tcl_config
from pprint import pprint
from time import sleep
import re
import argparse
import sys
from datetime import datetime, timedelta
import logging

def main():
  parser = argparse.ArgumentParser(description='Process some integers.')

  parser.add_argument('--prod', action='store_true')
  parser.add_argument('--remove-all-pms', action='store_true')

  args = parser.parse_args()
  conf = parse_tcl_config("~/.ritz.tcl")
  pprint(conf)

  if args.prod:
    c_server = conf["default"]["Server"]
    c_user   = conf["default"]["User"]
    c_secret = conf["default"]["Secret"]
  else:
    c_server = conf["UNINETT-backup"]["Server"]
    c_user   = conf["UNINETT-backup"]["User"]
    c_secret = conf["UNINETT-backup"]["Secret"]

  logging.basicConfig()
  logging.getLogger().setLevel(logging.DEBUG)
  requests_log = logging.getLogger("socket")
  requests_log.setLevel(logging.DEBUG)
  requests_log.propagate = True






  sess = ritz(c_server)
  sess.connect()
  sess.authenticate(c_user, c_secret)

  print("List all PMs:")
  pm = sess.pm_list()
  print(pm)
  #for i in pm:
  #  print("canceling %d" % i)
  #  sess.pm_cancel(i)

  print("Schedule test pm:")
  pm = sess.pm_add_device(datetime.now()+timedelta(minutes=1),
                          datetime.now()+timedelta(minutes=2),
                          "teknobyen-gw*", m_type='str')
  print("Scheduled")

  print("List all PMs:")
  pms = sess.pm_list()
  print(pm)

  print("pm_get_details:")
  print(sess.pm_get_details(pm))

  print("pm_get_log:")
  print(sess.pm_get_log(pm))

  print("pm_add_log:")
  sess.pm_add_log(pm, "This is a test log :)")

  print("pm_get_details:")
  print(sess.pm_get_details(pm))

  print("pm_get_log:")
  print(sess.pm_get_log(pm))

  print("pm_get_matching:")
  print(sess.pm_get_matching(pm))

  print("pm_cancel:")
  print(sess.pm_cancel(pm))


  return






if __name__ == "__main__":
  main()



import subprocess
import re
import sys
import os
from ritz import ritz
from datetime import datetime, timedelta


def main():
    """
    Script to schedule downtime of interfaces in zino when a server reboots.

    This script tries to detect interface names via LLDP and uses the env
    variables ZINOUSER and ZINOSECRET to connect to zino as as a ritz client.
    """
    if "ZINOUSER" in os.environ and "ZINOSECRET" in os.environ:
        user = os.environ["ZINOUSER"]
        secret = os.environ["ZINOSECRET"]
    else:
        sys.exit("ZINOUSER and/or ZINOSECRET env variable is not set, exiting")

    if "ZINOSERVER" in os.environ:
        server = os.environ["ZINOSERVER"]
    else:
        server = "zino.uninett.no"

    ports = collect_interfaces()
    if ports:
        with ritz(server, username=user, password=secret) as s:
            for device, porttype, port in ports:
                try:
                    id = s.pm_add_interface(datetime.now() + timedelta(seconds=15),
                                            datetime.now() + timedelta(minutes=30),
                                            device,
                                            r"^{}(\.\d+)?$".format(port))
                    print("Scheduled maintenance for %s %s : id %s" % (device, port, id))
                except Exception as e:
                    print("Unable to schedule maintenance for %s %a : %s" (device, port, e))


def collect_interfaces():
    lldp = subprocess.check_output(["lldpctl"])

    portlist = []

    device = ""
    print("Locating interfaces:")
    for line in lldp.decode().split("\n"):
        d = re.search("SysName:(.*)$", line)
        if d:
            device = d.group(1).strip()

        p = re.search("PortID:\W+(\w+)\W+(\S+)$", line)
        if p:
            porttype, port = p.groups()
            if porttype not in ["local", "ifname"]:
                print("Unable to add interface %s %s, Type needs to be ifname, not %s" % (device, port, porttype))
            else:
                portlist.append([device, porttype, port])
                print("Found interface: %s %s" % (device, port))


    return portlist





if __name__ == "__main__":
    main()



import subprocess
import logging
import re
import sys
import os
import socket
from ritz import ritz
from datetime import datetime, timedelta




def main():
    """
    Script to schedule downtime of interfaces in zino when a server reboots.

    This script tries to detect interface names via LLDP and uses the env
    variables ZINOUSER and ZINOSECRET to connect to zino as as a ritz client.
    """
    if "ZINODEBUG" in os.environ:
        ritzlog = logging.getLogger("ritz")
        ritzlog.setLevel(logging.DEBUG)
        ritzlog.addHandler(logging.FileHandler('comm.log'))

    if "ZINOUSER" in os.environ and "ZINOSECRET" in os.environ:
        user = os.environ["ZINOUSER"]
        secret = os.environ["ZINOSECRET"]
    else:
        sys.exit("ZINOUSER and/or ZINOSECRET env variable is not set, exiting")

    if "ZINOSERVER" in os.environ:
        server = os.environ["ZINOSERVER"]
    else:
        server = "zino.uninett.no"
    if "ZINOHOSTNAME" in os.environ:
        fqdn = os.environ["ZINOHOSTNAME"]
    else:
        fqdn = socket.getfqdn()

    if not fqdn.endswith(".uninett.no"):
        sys.exit("Wooth? sysname does not end with uninett.no?")

    print("Creating Maintenance on physical interfaces")
    with ritz(server, username=user, password=secret) as s:
        try:
            # Actually searching for ', fqdn.domain.ext'
            pm1 = s.pm_add_interface_bydescr(datetime.now() + timedelta(seconds=5),
                                             datetime.now() + timedelta(minutes=30),
                                             r',\s%s' % fqdn.replace(r'.',r'\.'))
            pm1matching = s.pm_get_matching(pm1)

            for i in pm1matching:
                print("Scheduled maintenance for %s %s %s" % (i[1], i[3], pm1))
        except Exception as e:
            print("Unable to schedule maintenance: %s" % (e))

if __name__ == "__main__":
    main()
